from django.core.exceptions import ValidationError
from django.core.files.uploadedfile import SimpleUploadedFile
from django.db import models
from django.utils.text import slugify
from django.utils.translation import gettext as _
from io import BytesIO
from pdf2image import convert_from_bytes
from uuid import uuid4

from doma.settings import DOCUMENTS_DIR, PREVIEW_DIR, RENDER_PREVIEW


class CreatedModifiedModel(models.Model):
    updated_at = models.DateTimeField(auto_now=True)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True


class SlugifiedModel(models.Model):
    slug = models.SlugField(editable=False, unique=True)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self._state.adding:
            self.slug = slugify(self.name)
        super().save(*args, **kwargs)


class DocumentType(SlugifiedModel):
    name = models.CharField(max_length=255)


class DocumentManager(models.Manager):
    def replace(self, document, file):
        """
        Even though django recommends to just set the pk to None to duplicate
        an object, we cannot do this here due to the various hooks implemented
        in the init and save methods of Document.
        """
        new_document = self.create(
            name=document.name,
            type=document.type,
            preview=document.preview,
            content=document.content,
            replaces=document,
            file=file,
        )
        return new_document


class Document(CreatedModifiedModel):
    """
    The default ordering is by created_at (desc), because the contents of a document,
    are given to the renderer upon creation of the instance. After rendering it is
    updated (i.e. modified) but can already be out of date.
    """

    name = models.CharField(max_length=255)
    type = models.ForeignKey(
        "DocumentType", on_delete=models.PROTECT, related_name="documents"
    )
    uuid = models.UUIDField(unique=True, default=uuid4, editable=False)
    preview = models.ImageField(upload_to=PREVIEW_DIR, null=True, blank=True)
    content = models.TextField(null=True, blank=True)
    replaces = models.OneToOneField(
        "Document",
        on_delete=models.PROTECT,
        related_name="replaced_by",
        null=True,
        blank=True,
    )
    file = models.FileField(upload_to=DOCUMENTS_DIR, blank=True)

    objects = DocumentManager()

    __file = None

    class Meta:
        ordering = ["-created_at"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.__file = self.file

    def __str__(self):
        return f"{self.pk} - {self.name}"

    def save(self, *args, **kwargs):
        if self.pk and self.file != self.__file:
            raise ValidationError(_("Files are not editable."))
        if not self.preview and RENDER_PREVIEW:
            preview = BytesIO()
            convert_from_bytes(self.file.read())[0].save(preview, "JPEG")
            self.preview = SimpleUploadedFile("preview.jpg", preview.read())
        super().save(*args, **kwargs)



# Generated by Django 3.2 on 2021-04-18 11:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("doma", "0002_alter_document_file"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="preview",
            field=models.ImageField(blank=True, null=True, upload_to="preview/"),
        ),
    ]



from django.apps import AppConfig
from django.db.models.signals import pre_delete
from doma.signals import protect_documents


class DomaConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "doma"

    def ready(self):
        pre_delete.connect(
            protect_documents, sender="doma.Document", dispatch_uid="delete_document"
        )



# Generated by Django 3.2 on 2021-04-18 10:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("doma", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="file",
            field=models.FileField(blank=True, upload_to="documents/"),
        ),
    ]



from django.db.models import ProtectedError
from django.utils import timezone
from django.utils.translation import gettext as _

from doma.settings import PROTECT_AFTER


def protect_documents(sender, instance, **kwargs):
    """
    This signals prevents documents older that PROTECT_AFTER (default: 1 day) from deletion.
    """
    if timezone.now() - instance.created_at > PROTECT_AFTER:
        raise ProtectedError(instance, _("This document cannot be deleted."))



# Generated by Django 3.2 on 2021-04-18 14:43

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("doma", "0003_alter_document_preview"),
    ]

    operations = [
        migrations.AlterField(
            model_name="document",
            name="replaces",
            field=models.OneToOneField(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="replaced_by",
                to="doma.document",
            ),
        ),
    ]



# Generated by Django 3.2 on 2021-04-17 14:49

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="DocumentType",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("slug", models.SlugField(editable=False, unique=True)),
                ("name", models.CharField(max_length=255)),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Document",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("updated_at", models.DateTimeField(auto_now=True)),
                ("created_at", models.DateTimeField(auto_now_add=True)),
                ("name", models.CharField(max_length=255)),
                (
                    "uuid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("preview", models.ImageField(blank=True, null=True, upload_to="")),
                ("content", models.TextField(blank=True, null=True)),
                (
                    "file",
                    models.FileField(blank=True, null=True, upload_to="documents/"),
                ),
                (
                    "replaces",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="replaced_by",
                        to="doma.document",
                    ),
                ),
                (
                    "type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="documents",
                        to="doma.documenttype",
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
            },
        ),
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': '.'}

packages = \
['doma', 'doma.migrations']

package_data = \
{'': ['*']}

install_requires = \
['Django>=3.2,<4.0',
 'Pillow>=8.2.0,<9.0.0',
 'freezegun>=1.1.0,<2.0.0',
 'pdf2image>=1.14.0,<2.0.0']

setup_kwargs = {
    'name': 'django-doma',
    'version': '0.2.2',
    'description': 'Simple Document Management for Django',
    'long_description': '# doma\n\n**doma** is a simple document managment app for django.\nIt provides the necessary models and view and is ready to be included into your project.\n\nDetailed documentation is in the "docs" directory.\n\n## Compatibility\n\nTested with the following versions of Python/Django:\n\n* Django: 2.2, 3.0,, 3.1, 3.2\n* Python: 3.7, 3.8, 3.9\n* Pypy: Pypy3\n\n## Installation\n\nInstall `django-doma` using pip:\n\n```zsh\n$ pip install django-doma\n```\n\n## Quick start\n\n1. Add "doma" to your INSTALLED_APPS setting like this::\n\n```python\nINSTALLED_APPS = [\n    ...\n    "doma",\n]\n```\n\n2. Include the polls URLconf in your project urls.py like this::\n\n    path(\'doma/\', include(\'doma.urls\')),\n\n3. Run ``python manage.py migrate`` to create the doma models.\n\n4. Visit http://127.0.0.1:8000/doma/ to start accounting.\n',
    'author': 'Florian Rämisch',
    'author_email': 'olf@subsignal.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/olf42/django-doma',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from datetime import timedelta
from django.conf import settings

DOCUMENTS_DIR = getattr(settings, "DOMA_DOCUMENTS_DIR", "documents/")

PREVIEW_DIR = getattr(settings, "DOMA_PREVIEW_DIR", "preview/")

PROTECT_AFTER = getattr(settings, "DOMA_PROTECT_AFTER", timedelta(days=1))

RENDER_PREVIEW = getattr(settings, "DOMA_RENDER_PREVIEW", True)
